#include <math.h>

class Screen_Element {
public:
	int x;
	int y;
	int w;
	int h;
	char *tooltip;
	bool focused;
	bool enabled;
	virtual void enable(){enabled=true;};
	virtual void draw()=0;
	virtual bool mouseinput(int mx, int my, bool mb1) {return false;};//returns true if wants to capture the mouse
	bool isat(int px, int py);
	Screen_Element(): focused(false) {};
	Screen_Element(int ex, int ey, int ew, int eh, char *etooltip,bool eenabled): x(ex), y(ey), w(ew), h(eh), tooltip(etooltip),focused(false),enabled(eenabled){};
	virtual ~Screen_Element() {};
};

class Date_Display: public Screen_Element{
public:
	void draw();
	Date_Display(int ex, int ey, int ew, int eh, char* etooltip):
	Screen_Element(ex,ey,ew,eh,etooltip,true)
	{};
	~Date_Display(){};
};

const char month_text[12][4]={"jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};
const char year_text[6][3]={"39","40","41","42","43","44"};

void Date_Display::draw()
{

	
	for (int i=0;i<3;i++)
	{
			masked_blit(fontbmp[chartonum(month_text[warsaw->month][i])],buffer,0,0,x+1+4*i,y+1,3,5);
	}
	
	for (int i=0;i<2;i++)
	{
		masked_blit(fontbmp[chartonum(year_text[warsaw->year][i])],buffer,0,0,x+1+4*i+4*4,y+1,3,5);
	}
		
}

class MapView: public Screen_Element {
public:
	void draw();
	MapView(int ex, int ey, int ew, int eh, char *etooltip):
	Screen_Element(ex,ey,ew,eh,etooltip,true)
	{};
	~MapView(){};
};

void MapView::draw()
{
	if (warsaw->ghetto)
	{
		static int ghettotimer=0;
		if (ghettotimer<255*4)
		{
			ghettotimer++;
			set_trans_blender(255, 255, 255, ghettotimer/4);
			draw_trans_sprite(buffer, ghetto2, 82, 22);
		}
		else
		{
			if (warsaw->wall)
			{
				static int walltimer=0;
				if (walltimer<255*4)
				{
					walltimer++;
					blit(ghetto2, buffer, 0,0,82, 22, 17, 18);
					set_trans_blender(255, 255, 255, walltimer/4);
					draw_trans_sprite(buffer, ghetto, 82, 22);
				}
				else
				{
					blit(ghetto, buffer, 0,0,82, 22, 17, 18);
				}
			}
			else
				blit(ghetto2, buffer, 0,0,82, 22, 17, 18);
		}
	}
	
	for (int i=0;i<warsaw->totalresistance*50;i++)
	{
		if (changeflame)
		{
			flame[i][2]=flamecycle[rand()%flamecyclelength];
		}
		putpixel(buffer,flame[i][0],flame[i][1],flame[i][2]);
	}
	std::cout<<warsaw->totalresistance*50<<"\n";
	changeflame=false;
}

class Status_Bar: public Screen_Element {
public:
	void draw();
	Status_Bar(int ex, int ey, int ew, int eh, char *etooltip):
	Screen_Element(ex,ey,ew,eh,etooltip,true)
	{};
//	~Status_Bar=0;
};

void Status_Bar::draw()
{	
	int i=0;
	if (emergencytimer==0)
	{
		{
			while (statustext[i]!=0)
			{
				if (statustext[i]!=' ')
					masked_blit(fontbmp[chartonum(statustext[i])],buffer,0,0,x+1+4*i,y+1,3,5);
				i++;
			}
		}
	}
	else if ((emergencytimer/30)%2==0)
	{
		
		const char* text=emergencymessages[0].c_str();
		while (text[i]!=0)
		{
			if (text[i]!=' ')
				masked_blit(redfontbmp[chartonum(text[i])],buffer,0,0,x+1+4*i,y+1,3,5);
			i++;
		}
	}
}

bool Screen_Element::isat(int px, int py)
{
	if (((px>=x)  &&  (px<x+w))&&((py>=y)&&(py<y+h)))
		return true;
	else
		return false;
}

class Graphic_Element :public Screen_Element {
public:
	BITMAP *img;
	int imgw;//no imageheight for now ;)
	Graphic_Element(int ex, int ey, int ew, int eh, int eimgw, BITMAP *eimage, char *etooltip,bool eenabled):
	Screen_Element(ex,ey,ew,eh,etooltip,eenabled), 
	img(eimage), imgw(eimgw) {};
	virtual void draw()=0;
};

class Singlebar:public Graphic_Element {
public:
	num* target;
	void draw();
	Singlebar(int ex, int ey, int ew, int eh, int eimgw, BITMAP *eimage, char *etooltip, num* etarget):
	Graphic_Element(ex, ey, ew, eh, eimgw, eimage, etooltip,true), 
	target(etarget){};
};

void Singlebar::draw() {
	int amount=round((*target)*28.0);
	blit(img,buffer,0,0,x,y,imgw,h);
	rect(buffer,x,y,x+w-1,y+h-1,makecol(0,0,0));
	//            rect(buffer,x+imgw-1,y,x+imgw+amount1,y+h-1,makecol(0,0,0));
	
	if (amount>0)
	{
		rectfill(buffer,x+imgw,y+1,x+imgw+amount-1,y+h-2,makecol(255,255,0));
		vline(buffer,x+imgw+amount-1,y+1,y+h-2,makecol(0,0,0));
	}
}


class Doublebar:public Graphic_Element {
public:
	num* targets;
	void draw();
	Doublebar(int ex, int ey, int ew, int eh, int eimgw, BITMAP *eimage, char *etooltip, num* etargets):
	Graphic_Element(ex, ey, ew, eh, eimgw, eimage, etooltip,true), 
	targets(etargets)
	{};
};

void Doublebar::draw() {
	blit(img,buffer,0,0,x,y,imgw,h);
	rect(buffer,x,y,x+w-1,y+h-1,makecol(0,0,0));
	//            rect(buffer,x+imgw-1,y,x+imgw+amount1,y+h-1,makecol(0,0,0));
	
	int amount1=round((*targets)*27.0);
	int amount2=amount1+round((*(targets+1))*26.0);
					  
	if (amount1>1)
	{
		rectfill(buffer,x+imgw,y+1,x+imgw+amount1-2,y+h-2,makecol(255,0,0));
		vline(buffer,x+imgw+amount1-1,y+1,y+h-2,makecol(0,0,0));
	}
	if (amount2>amount1)
	{
		rectfill(buffer,x+imgw-1+amount1,y,x+imgw+amount2,y+h-1,makecol(255,255,0));
		rect(buffer,x+imgw-1+amount1,y,x+imgw+amount2,y+h-1,makecol(0,0,0));
	}
	//         rectfill(buffer,x+imgw-1,y,x+imgw+amount1,y+h-1,makecol(255,0,0));
	//       rect(buffer,x+imgw-1,y,x+imgw+amount1,y+h-1,makecol(0,0,0));			
	//     rectfill(buffer,x+imgw-1+amount1,y,x+imgw+amount2,y+h-1,makecol(255,255,0));
	//   rect(buffer,x+imgw-1+amount1,y,x+imgw+amount2,y+h-1,makecol(0,0,0));
}


class Inputbar:public Graphic_Element {
public:
	int amount;
	num* target;
	void draw();
	bool mouseinput(int mx, int my, bool mb1);
	Inputbar(int ex, int ey, int ew, int eh, int eimgw, BITMAP *eimage, char *etooltip, int eamount,num* etarget):
	Graphic_Element(ex, ey, ew, eh, eimgw, eimage, etooltip,true), 
	amount(eamount),
	target(etarget)
	{};
};

void Inputbar::draw()
{

	rect(buffer,x,y,x+w-1,y+h-1,makecol(0,0,0));
	blit(img,buffer,0,0,x,y,imgw,h);
	
	if (amount>0)
	{
		rectfill(buffer,x+imgw,y+1,x+imgw+amount-1,y+h-2,makecol(255,255,0));
		vline(buffer,x+imgw+amount-1,y+1,y+h-2,makecol(0,0,0));
	}
	
	//       rect(buffer,x+imgw-1,y,x+imgw+amount-1,y+h-1,makecol(0,0,0));
	
	//       rect(buffer,x+imgw-1,y,x+imgw+amount,y+h-1,makecol(0,0,0));
}

bool Inputbar::mouseinput(int mx, int my, bool mb1)
{
	if (mb1)
	{
		if (mx>x+imgw-2)
		{
			if (mx<x+w-1)
				amount=mx-x-imgw+1;
			else
				amount=w-imgw;
		}
		else amount=0;

		*target=(num)amount/((num)37.0);//37 is the max; scaling to have it be between zero and one
		return true;
	}
	else return false;
}


enum Button_State {bs_normal,mouse_down,selected,disabled};

//for this game, don't need to allow turn off after turning on
class Set_Button : public Screen_Element {
public:
	Button_State state;
	BITMAP *graphicarray[4];
	bool on;
	bool toggleable;
	bool* target;
	void draw();
	void enable();
	bool mouseinput(int mx, int my, bool mb1);
	Set_Button(int ex, int ey, int ew, int eh, BITMAP* images, char* etooltip, bool eenabled,bool etoggleable, bool* etarget):
	Screen_Element(ex, ey, ew, eh, etooltip,eenabled),
	on(false),
	toggleable(etoggleable),
	target(etarget)
	{ 
		for (int i=0;i<4;i++)
		{
			graphicarray[i]=create_bitmap(w,h);
			blit(images,graphicarray[i],0,i*h,0,0,w,h);
		}
		if (enabled) 
			state=bs_normal;
		else	state=disabled;
	};	
};

void Set_Button::enable()
{
	if (enabled==false)
	{
		enabled=true;
		state=bs_normal;
	};
}

void Set_Button::draw()
{
	if ( !toggleable || focused || state==selected)
		blit(graphicarray[state],buffer,0,0,x,y,w,h);
	else
		blit(graphicarray[state],buffer,1,1,x+1,y+1,w-2,h-2);
	
}

bool Set_Button::mouseinput(int mx, int my, bool mb1)
{
	if (state!=selected)
	{
		//if the mouse is over it
		if ((state!=disabled))
		{
			//if (onover!=NULL)
			//	onover();
			//if somebody presses the mouse button and the button is not being held
			if (mb1&&(state!=mouse_down))
			{
				//then hold it;
				state=mouse_down;
				return true;
			}
			//else if it was being held and someone lets go, change the state
			else if (!mb1&&(state==mouse_down))
			{
				if (isat(mx,my))
					on=!on;
				
				//select the correct image;
				if (on)
				{
					state=selected;
					*target=true;
					if (target==&warsaw->ghetto)//special code here
						enable_buttons();
				}
				else
					state=bs_normal;
				return false;
			}
			
			//and if it is unchecked
			else if (state==bs_normal)
			{
				//if somebody presses the left mousebutton
				if (mb1)
				{//change it's state to being down
					state=mouse_down;
					return true;
				}
			}
			else
			{
				
				//	state=bs_normal;
				if (mb1)
					return true;
				else
				{
					
					if (on)
						state=selected;
					else
						state=bs_normal;
					return false;
				}
			}
		}
		else
		{
			if (on)
				state=selected;
			else
				state=bs_normal;
			return false;
		}
	}
	
	return false;
}


//for this game, don't need to allow turn off after turning on
class Toggle_Button : public Screen_Element {
public:
	Button_State state;
	BITMAP *graphicarray[4];
	bool on;
	bool toggleable;
	bool* target;
	void draw();
	void enable();
	bool mouseinput(int mx, int my, bool mb1);
	Toggle_Button(int ex, int ey, int ew, int eh, BITMAP* images, char* etooltip, bool eenabled,bool etoggleable, bool* etarget):
	Screen_Element(ex, ey, ew, eh, etooltip,eenabled),
	on(false),
	toggleable(etoggleable),
	target(etarget)
	{ 
		for (int i=0;i<4;i++)
		{
			graphicarray[i]=create_bitmap(w,h);
			blit(images,graphicarray[i],0,i*h,0,0,w,h);
		}
		if (enabled) 
			state=bs_normal;
		else	state=disabled;
	};	
};

void Toggle_Button::enable()
{
	if (enabled==false)
	{
		enabled=true;
		state=bs_normal;
	};
}

void Toggle_Button::draw()
{
	if ( !toggleable || focused || state==selected)
		blit(graphicarray[state],buffer,0,0,x,y,w,h);
	else
		blit(graphicarray[state],buffer,1,1,x+1,y+1,w-2,h-2);
	
}

bool Toggle_Button::mouseinput(int mx, int my, bool mb1)
{

	//if the mouse is over it
	if ((state!=disabled))
	{
		//if (onover!=NULL)
		//	onover();
		//if somebody presses the mouse button and the button is not being held
		if (mb1&&(state!=mouse_down))
		{
			//then hold it;
			state=mouse_down;
			return true;
		}
		//else if it was being held and someone lets go, change the state
		else if (!mb1&&(state==mouse_down))
		{
			if (isat(mx,my))
				on=!on;
			
			*target=on;
			//select the correct image;
			if (on)
			{
				state=selected;
			}
			else
				state=bs_normal;
			return false;
		}
		
		//and if it is unchecked
		else if (state==bs_normal)
		{
			//if somebody presses the left mousebutton
			if (mb1)
			{//change it's state to being down
				state=mouse_down;
				return true;
			}
		}
		else
		{
			
			//	state=bs_normal;
			if (mb1)
				return true;
			else
			{
				
				if (on)
					state=selected;
				else
					state=bs_normal;
				return false;
			}
		}
	}
	else
	{
		if (on)
			state=selected;
		else
			state=bs_normal;
		return false;
	}
	
	return false;
}




class Button:public Screen_Element {
public:
	Button_State state;
	BITMAP *graphicarray[4];
	bool on;
	bool toggleable;
	void draw();
	bool mouseinput(int mx, int my, bool mb1);
	Button(int ex, int ey, int ew, int eh, BITMAP* images, char* etooltip, bool eenabled,bool etoggleable):
		Screen_Element(ex, ey, ew, eh, etooltip,eenabled),
		on(false),toggleable(etoggleable)
	{ 
		for (int i=0;i<4;i++)
		{
			graphicarray[i]=create_bitmap(w,h);
			blit(images,graphicarray[i],0,i*h,0,0,w,h);
		}
		if (enabled) 
			state=bs_normal;
		else	state=disabled;
	};
};

void Button::draw()
{
	if ( !toggleable || focused || state==selected)
		blit(graphicarray[state],buffer,0,0,x,y,w,h);
	else
		blit(graphicarray[state],buffer,1,1,x+1,y+1,w-2,h-2);

}

bool Button::mouseinput(int mx, int my, bool mb1)
{
	
	//if the mouse is over it
	if ((state!=disabled))
	{
		//if (onover!=NULL)
		//	onover();
		//if somebody presses the mouse button and the button is not being held
		if (mb1&&(state!=mouse_down))
		{
			//then hold it;
			state=mouse_down;
			return true;
		}
		//else if it was being held and someone lets go, change the state
		else if (!mb1&&(state==mouse_down))
		{
			if (isat(mx,my))
				on=!on;
			
			//select the correct image;
			if (on)
				state=selected;
			else
				state=bs_normal;
			return false;
		}
		
		//and if it is unchecked
		else if (state==bs_normal)
		{
			//if somebody presses the left mousebutton
			if (mb1)
			{//change it's state to being down
				state=mouse_down;
				return true;
			}
		}
		else
		{
			
			//	state=bs_normal;
			if (mb1)
				return true;
			else
			{
				
				if (on)
					state=selected;
				else
					state=bs_normal;
				return false;
			}
		}
	}
	else
	{
		if (on)
			state=selected;
		else
			state=bs_normal;
		return false;
	}

	return false;
}


